package fr.contralia.client.example;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

// Homemade XmlReader which simplifies XML manipulation
public class XmlReader {
	private XmlReader() {
	}

	public static Node2 parse(String xml) throws Exception {
		try {
			DocumentBuilderFactory f = DocumentBuilderFactory.newInstance();
			f.setNamespaceAware(true);
			InputStream input = new ByteArrayInputStream(xml.getBytes());
			Document document = f.newDocumentBuilder().parse(input);
			return new XmlReader.Node2(getRootNode(document));
		} catch (Exception e) {
			int maxChars = Math.min(xml.length(), 1000);
			throw new Exception("Unable to parse XML starting with '" + xml.substring(0, maxChars) + "'", e);
		}
	}

	private static Node getRootNode(Document document) {
		Node node;
		for(node = document.getFirstChild(); node != null && node.getNodeType() != 1; node = node.getNextSibling()) {
		}
		return node;
	}

	public static class Node2 {
		Node node;

		public Node2(Node node) {
			this.node = node;
		}

		public boolean isNull() {
			return this.node == null;
		}

		public String getText() {
			if (this.node == null) {
				return null;
			} else {
				StringBuilder sb = new StringBuilder();

				for(int i = 0; i < this.node.getChildNodes().getLength(); ++i) {
					Node child = this.node.getChildNodes().item(i);
					if (child.getNodeType() == Node.TEXT_NODE) {
						sb.append(child.getNodeValue());
					}
				}

				return sb.toString().trim();
			}
		}

		public String getName() {
			return this.node == null ? null : this.node.getLocalName();
		}

		public String getAttribute(String attribName) {
			if (this.node == null) {
				return null;
			} else {
				Node attribNode = this.node.getAttributes().getNamedItem(attribName);
				return attribNode == null ? null : attribNode.getNodeValue();
			}
		}

		public XmlReader.Node2 getChild(String nodeName) {
			if (this.node == null) {
				return new XmlReader.Node2(null);
			} else {
				for(int i = 0; i < this.node.getChildNodes().getLength(); ++i) {
					Node child = this.node.getChildNodes().item(i);
					if (nodeName.equals(child.getLocalName())) {
						return new XmlReader.Node2(child);
					}
				}

				return new XmlReader.Node2(null);
			}
		}

		public List<XmlReader.Node2> getChildren(String nodeName) {
			List<XmlReader.Node2> list = new ArrayList();
			if (this.node != null) {
				for(int i = 0; i < this.node.getChildNodes().getLength(); ++i) {
					Node child = this.node.getChildNodes().item(i);
					if (nodeName.equals(child.getLocalName())) {
						list.add(new XmlReader.Node2(child));
					}
				}
			}

			return list;
		}
	}
}
